@@ -3,30 +3,48 @@ package chezmoi
 import (
 	"bytes"
 	"encoding/hex"
+	"fmt"
 	"io/fs"
+	"os/exec"
 	"runtime"
 	"time"
 )
 
 // A TargetStateEntry represents the state of an entry in the target state.
 type TargetStateEntry interface {
-	Apply(system System, persistentState PersistentState, actualStateEntry ActualStateEntry) (bool, error)
+	Apply(
+		system System,
+		persistentState PersistentState,
+		actualStateEntry ActualStateEntry,
+	) (bool, error)
 	EntryState(umask fs.FileMode) (*EntryState, error)
 	Evaluate() error
 	SkipApply(persistentState PersistentState, targetAbsPath AbsPath) (bool, error)
+	SourceAttr() SourceAttr
+}
+
+// A TargetStateModifyDirWithCmd represents running a command that modifies
+// a directory.
+type TargetStateModifyDirWithCmd struct {
+	cmd           *exec.Cmd
+	forceRefresh  bool
+	refreshPeriod Duration
+	sourceAttr    SourceAttr
 }
 
 // A TargetStateDir represents the state of a directory in the target state.
 type TargetStateDir struct {
-	perm fs.FileMode
+	perm       fs.FileMode
+	sourceAttr SourceAttr
 }
 
 // A TargetStateFile represents the state of a file in the target state.
 type TargetStateFile struct {
 	*lazyContents
-	empty     bool
-	overwrite bool
-	perm      fs.FileMode
+	empty      bool
+	overwrite  bool
+	perm       fs.FileMode
+	sourceAttr SourceAttr
 }
 
 // A TargetStateRemove represents the absence of an entry in the target state.
@@ -38,24 +56,97 @@ type TargetStateScript struct {
 	name        RelPath
 	interpreter *Interpreter
 	condition   ScriptCondition
+	sourceAttr  SourceAttr
 }
 
 // A TargetStateSymlink represents the state of a symlink in the target state.
 type TargetStateSymlink struct {
 	*lazyLinkname
+	sourceAttr SourceAttr
 }
 
-// A targetStateRenameDir represents the renaming of a directory in the target
-// state.
-type targetStateRenameDir struct {
-	oldRelPath RelPath
-	newRelPath RelPath
+// A modifyDirWithCmdState records the state of a directory modified by a
+// command.
+type modifyDirWithCmdState struct {
+	Name  AbsPath   `json:"name"  yaml:"name"`
+	RunAt time.Time `json:"runAt" yaml:"runAt"`
 }
 
 // A scriptState records the state of a script that has been run.
 type scriptState struct {
-	Name  RelPath   `json:"name" toml:"name" yaml:"name"`
-	RunAt time.Time `json:"runAt" toml:"runAt" yaml:"runAt"`
+	Name  RelPath   `json:"name"  yaml:"name"`
+	RunAt time.Time `json:"runAt" yaml:"runAt"`
+}
+
+// Apply updates actualStateEntry to match t.
+func (t *TargetStateModifyDirWithCmd) Apply(
+	system System, persistentState PersistentState, actualStateEntry ActualStateEntry,
+) (bool, error) {
+	if _, ok := actualStateEntry.(*ActualStateDir); !ok {
+		if err := actualStateEntry.Remove(system); err != nil {
+			return false, err
+		}
+	}
+
+	runAt := time.Now().UTC()
+	if err := system.RunCmd(t.cmd); err != nil {
+		return false, fmt.Errorf("%s: %w", actualStateEntry.Path(), err)
+	}
+
+	modifyDirWithCmdStateKey := []byte(actualStateEntry.Path().String())
+	if err := PersistentStateSet(
+		persistentState, GitRepoExternalStateBucket, modifyDirWithCmdStateKey, &modifyDirWithCmdState{
+			Name:  actualStateEntry.Path(),
+			RunAt: runAt,
+		}); err != nil {
+		return false, err
+	}
+
+	return true, nil
+}
+
+// EntryState returns t's entry state.
+func (t *TargetStateModifyDirWithCmd) EntryState(umask fs.FileMode) (*EntryState, error) {
+	return &EntryState{
+		Type: EntryStateTypeDir,
+		Mode: fs.ModeDir | fs.ModePerm&^umask,
+	}, nil
+}
+
+// Evaluate evaluates t.
+func (t *TargetStateModifyDirWithCmd) Evaluate() error {
+	return nil
+}
+
+// SkipApply implements TargetStateEntry.SkipApply.
+func (t *TargetStateModifyDirWithCmd) SkipApply(
+	persistentState PersistentState,
+	targetAbsPath AbsPath,
+) (bool, error) {
+	if t.forceRefresh {
+		return false, nil
+	}
+	modifyDirWithCmdKey := []byte(targetAbsPath.String())
+	switch modifyDirWithCmdStateBytes, err := persistentState.Get(GitRepoExternalStateBucket, modifyDirWithCmdKey); {
+	case err != nil:
+		return false, err
+	case modifyDirWithCmdStateBytes == nil:
+		return false, nil
+	default:
+		var modifyDirWithCmdState modifyDirWithCmdState
+		if err := stateFormat.Unmarshal(modifyDirWithCmdStateBytes, &modifyDirWithCmdState); err != nil {
+			return false, err
+		}
+		if t.refreshPeriod == 0 {
+			return true, nil
+		}
+		return time.Since(modifyDirWithCmdState.RunAt) < time.Duration(t.refreshPeriod), nil
+	}
+}
+
+// SourceAttr implements TargetStateEntry.SourceAttr.
+func (t *TargetStateModifyDirWithCmd) SourceAttr() SourceAttr {
+	return t.sourceAttr
 }
 
 // Apply updates actualStateEntry to match t. It does not recurse.
@@ -88,10 +179,18 @@ func (t *TargetStateDir) Evaluate() error {
 }
 
 // SkipApply implements TargetState.SkipApply.
-func (t *TargetStateDir) SkipApply(persistentState PersistentState, targetAbsPath AbsPath) (bool, error) {
+func (t *TargetStateDir) SkipApply(
+	persistentState PersistentState,
+	targetAbsPath AbsPath,
+) (bool, error) {
 	return false, nil
 }
 
+// SourceAttr implements TargetStateEntry.SourceAttr.
+func (t *TargetStateDir) SourceAttr() SourceAttr {
+	return t.sourceAttr
+}
+
 // Apply updates actualStateEntry to match t.
 func (t *TargetStateFile) Apply(
 	system System, persistentState PersistentState, actualStateEntry ActualStateEntry,
@@ -165,11 +264,19 @@ func (t *TargetStateFile) Perm(umask fs.FileMode) fs.FileMode {
 	return t.perm &^ umask
 }
 
-// SkipApply implements TargetState.SkipApply.
-func (t *TargetStateFile) SkipApply(persistentState PersistentState, targetAbsPath AbsPath) (bool, error) {
+// SkipApply implements TargetStateEntry.SkipApply.
+func (t *TargetStateFile) SkipApply(
+	persistentState PersistentState,
+	targetAbsPath AbsPath,
+) (bool, error) {
 	return false, nil
 }
 
+// SourceAttr implements TargetStateEntry.SourceAttr.
+func (t *TargetStateFile) SourceAttr() SourceAttr {
+	return t.sourceAttr
+}
+
 // Apply updates actualStateEntry to match t.
 func (t *TargetStateRemove) Apply(
 	system System, persistentState PersistentState, actualStateEntry ActualStateEntry,
@@ -192,11 +299,19 @@ func (t *TargetStateRemove) Evaluate() error {
 	return nil
 }
 
-// SkipApply implements TargetState.SkipApply.
-func (t *TargetStateRemove) SkipApply(persistentState PersistentState, targetAbsPath AbsPath) (bool, error) {
+// SkipApply implements TargetStateEntry.SkipApply.
+func (t *TargetStateRemove) SkipApply(
+	persistentState PersistentState,
+	targetAbsPath AbsPath,
+) (bool, error) {
 	return false, nil
 }
 
+// SourceAttr implements TargetStateEntry.SourceAttr.
+func (t *TargetStateRemove) SourceAttr() SourceAttr {
+	return SourceAttr{}
+}
+
 // Apply runs t.
 func (t *TargetStateScript) Apply(
 	system System, persistentState PersistentState, actualStateEntry ActualStateEntry,
@@ -220,13 +335,16 @@ func (t *TargetStateScript) Apply(
 	}
 	runAt := time.Now().UTC()
 	if !isEmpty(contents) {
-		if err := system.RunScript(t.name, actualStateEntry.Path().Dir(), contents, t.interpreter); err != nil {
+		if err := system.RunScript(t.name, actualStateEntry.Path().Dir(), contents, RunScriptOptions{
+			Condition:   t.condition,
+			Interpreter: t.interpreter,
+		}); err != nil {
 			return false, err
 		}
 	}
 
 	scriptStateKey := []byte(hex.EncodeToString(contentsSHA256))
-	if err := persistentStateSet(persistentState, scriptStateBucket, scriptStateKey, &scriptState{
+	if err := PersistentStateSet(persistentState, ScriptStateBucket, scriptStateKey, &scriptState{
 		Name:  t.name,
 		RunAt: runAt,
 	}); err != nil {
@@ -234,7 +352,7 @@ func (t *TargetStateScript) Apply(
 	}
 
 	entryStateKey := actualStateEntry.Path().Bytes()
-	if err := persistentStateSet(persistentState, EntryStateBucket, entryStateKey, &EntryState{
+	if err := PersistentStateSet(persistentState, EntryStateBucket, entryStateKey, &EntryState{
 		Type:           EntryStateTypeScript,
 		ContentsSHA256: HexBytes(contentsSHA256),
 	}); err != nil {
@@ -262,8 +380,17 @@ func (t *TargetStateScript) Evaluate() error {
 	return err
 }
 
-// SkipApply implements TargetState.SkipApply.
-func (t *TargetStateScript) SkipApply(persistentState PersistentState, targetAbsPath AbsPath) (bool, error) {
+// SkipApply implements TargetStateEntry.SkipApply.
+func (t *TargetStateScript) SkipApply(
+	persistentState PersistentState,
+	targetAbsPath AbsPath,
+) (bool, error) {
+	switch contents, err := t.Contents(); {
+	case err != nil:
+		return false, err
+	case len(contents) == 0:
+		return true, nil
+	}
 	switch t.condition {
 	case ScriptConditionAlways:
 		return false, nil
@@ -273,7 +400,7 @@ func (t *TargetStateScript) SkipApply(persistentState PersistentState, targetAbs
 			return false, err
 		}
 		scriptStateKey := []byte(hex.EncodeToString(contentsSHA256))
-		switch scriptState, err := persistentState.Get(scriptStateBucket, scriptStateKey); {
+		switch scriptState, err := persistentState.Get(ScriptStateBucket, scriptStateKey); {
 		case err != nil:
 			return false, err
 		case scriptState != nil:
@@ -301,6 +428,11 @@ func (t *TargetStateScript) SkipApply(persistentState PersistentState, targetAbs
 	return false, nil
 }
 
+// SourceAttr implements TargetStateEntry.SourceAttr.
+func (t *TargetStateScript) SourceAttr() SourceAttr {
+	return t.sourceAttr
+}
+
 // Apply updates actualStateEntry to match t.
 func (t *TargetStateSymlink) Apply(
 	system System, persistentState PersistentState, actualStateEntry ActualStateEntry,
@@ -362,32 +494,14 @@ func (t *TargetStateSymlink) Evaluate() error {
 	return err
 }
 
-// SkipApply implements TargetState.SkipApply.
+// SkipApply implements TargetStateEntry.SkipApply.
 func (t *TargetStateSymlink) SkipApply(
 	persistentState PersistentState, targetAbsPath AbsPath,
 ) (bool, error) {
 	return false, nil
 }
 
-// Apply renames actualStateEntry.
-func (t *targetStateRenameDir) Apply(
-	system System, persistentState PersistentState, actualStateEntry ActualStateEntry,
-) (bool, error) {
-	dir := actualStateEntry.Path().Dir()
-	return true, system.Rename(dir.Join(t.oldRelPath), dir.Join(t.newRelPath))
-}
-
-// EntryState returns t's entry state.
-func (t *targetStateRenameDir) EntryState(umask fs.FileMode) (*EntryState, error) {
-	return nil, nil
-}
-
-// Evaluate does nothing.
-func (t *targetStateRenameDir) Evaluate() error {
-	return nil
-}
-
-// SkipApply implements TargetState.SkipApply.
-func (t *targetStateRenameDir) SkipApply(persistentState PersistentState, targetAbsPath AbsPath) (bool, error) {
-	return false, nil
+// SourceAttr implements TargetStateEntry.SourceAttr.
+func (t *TargetStateSymlink) SourceAttr() SourceAttr {
+	return t.sourceAttr
 }
