@@ -574,7 +574,7 @@ func (c *Config) applyArgs(
 					return err
 				}
 			} else {
-				configStateValue, err := json.Marshal(configState{
+				configStateValue, err := chezmoi.FormatJSON.Marshal(configState{
 					ConfigTemplateContentsSHA256: chezmoi.HexBytes(currentConfigTemplateContentsSHA256),
 				})
 				if err != nil {
@@ -750,7 +750,7 @@ func (c *Config) createAndReloadConfigFile(cmd *cobra.Command) error {
 		return err
 	}
 
-	configStateValue, err := json.Marshal(configState{
+	configStateValue, err := chezmoi.FormatJSON.Marshal(configState{
 		ConfigTemplateContentsSHA256: chezmoi.HexBytes(chezmoi.SHA256Sum(configTemplate.contents)),
 	})
 	if err != nil {
@@ -899,6 +899,14 @@ func (c *Config) decodeConfigFile(configFileAbsPath chezmoi.AbsPath, configFile
 		return fmt.Errorf("%s: %w", configFileAbsPath, err)
 	}
 
+	if configFile.Git.CommitMessageTemplate != "" &&
+		configFile.Git.CommitMessageTemplateFile != "" {
+		return fmt.Errorf(
+			"%s: cannot specify both git.commitMessageTemplate and git.commitMessageTemplateFile",
+			configFileAbsPath,
+		)
+	}
+
 	return nil
 }
 
@@ -1440,35 +1448,11 @@ func (c *Config) gitAutoAdd() (*git.Status, error) {
 
 // gitAutoCommit commits all changes in the git index, including generating a
 // commit message from status.
-func (c *Config) gitAutoCommit(status *git.Status) error {
+func (c *Config) gitAutoCommit(cmd *cobra.Command, status *git.Status) error {
 	if status.Empty() {
 		return nil
 	}
-	funcMap := maps.Clone(sprig.TxtFuncMap())
-	maps.Copy(funcMap, map[string]any{
-		"promptBool":   c.promptBoolInteractiveTemplateFunc,
-		"promptChoice": c.promptChoiceInteractiveTemplateFunc,
-		"promptInt":    c.promptIntInteractiveTemplateFunc,
-		"promptString": c.promptStringInteractiveTemplateFunc,
-		"targetRelPath": func(source string) string {
-			return chezmoi.NewSourceRelPath(source).
-				TargetRelPath(c.encryption.EncryptedSuffix()).
-				String()
-		},
-	})
-	templateOptions := chezmoi.TemplateOptions{
-		Options: slices.Clone(c.Template.Options),
-	}
-	commitMessageTmpl, err := chezmoi.ParseTemplate(
-		"commit_message",
-		[]byte(c.Git.CommitMessageTemplate),
-		funcMap,
-		templateOptions,
-	)
-	if err != nil {
-		return err
-	}
-	commitMessage, err := commitMessageTmpl.Execute(status)
+	commitMessage, err := c.gitCommitMessage(cmd, status)
 	if err != nil {
 		return err
 	}
@@ -1487,6 +1471,63 @@ func (c *Config) gitAutoPush(status *git.Status) error {
 	return c.run(c.WorkingTreeAbsPath, c.Git.Command, []string{"push"})
 }
 
+// gitCommitMessage returns the git commit message for the given status.
+func (c *Config) gitCommitMessage(cmd *cobra.Command, status *git.Status) ([]byte, error) {
+	funcMap := maps.Clone(c.templateFuncs)
+	maps.Copy(funcMap, map[string]any{
+		"promptBool":   c.promptBoolInteractiveTemplateFunc,
+		"promptChoice": c.promptChoiceInteractiveTemplateFunc,
+		"promptInt":    c.promptIntInteractiveTemplateFunc,
+		"promptString": c.promptStringInteractiveTemplateFunc,
+		"targetRelPath": func(source string) string {
+			return chezmoi.NewSourceRelPath(source).
+				TargetRelPath(c.encryption.EncryptedSuffix()).
+				String()
+		},
+	})
+	var name string
+	var commitMessageTemplateData []byte
+	switch {
+	case c.Git.CommitMessageTemplate != "":
+		name = "git.commitMessageTemplate"
+		commitMessageTemplateData = []byte(c.Git.CommitMessageTemplate)
+	case c.Git.CommitMessageTemplateFile != "":
+		if c.sourceDirAbsPathErr != nil {
+			return nil, c.sourceDirAbsPathErr
+		}
+		commitMessageTemplateFileAbsPath := c.sourceDirAbsPath.JoinString(
+			c.Git.CommitMessageTemplateFile,
+		)
+		name = c.sourceDirAbsPath.String()
+		var err error
+		commitMessageTemplateData, err = c.baseSystem.ReadFile(commitMessageTemplateFileAbsPath)
+		if err != nil {
+			return nil, err
+		}
+	default:
+		name = "COMMIT_MESSAGE"
+		commitMessageTemplateData = []byte(templates.CommitMessageTmpl)
+	}
+	commitMessageTmpl, err := chezmoi.ParseTemplate(
+		name,
+		commitMessageTemplateData,
+		funcMap,
+		chezmoi.TemplateOptions{
+			Options: slices.Clone(c.Template.Options),
+		},
+	)
+	if err != nil {
+		return nil, err
+	}
+	sourceState, err := c.getSourceState(cmd.Context(), cmd)
+	if err != nil {
+		return nil, err
+	}
+	templateDataMap := sourceState.TemplateData()
+	templateDataMap["chezmoi"].(map[string]any)["status"] = status //nolint:forcetypeassert
+	return commitMessageTmpl.Execute(templateDataMap)
+}
+
 // makeRunEWithSourceState returns a function for
 // github.com/spf13/cobra.Command.RunE that includes reading the source state.
 func (c *Config) makeRunEWithSourceState(
@@ -1606,6 +1647,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		c.newDumpConfigCmd(),
 		c.newEditCmd(),
 		c.newEditConfigCmd(),
+		c.newEditConfigTemplateCmd(),
 		c.newEncryptCommand(),
 		c.newExecuteTemplateCmd(),
 		c.newForgetCmd(),
@@ -1772,7 +1814,7 @@ func (c *Config) persistentPostRunRootE(cmd *cobra.Command, args []string) error
 			}
 		}
 		if c.Git.AutoCommit || c.Git.AutoPush {
-			if err := c.gitAutoCommit(status); err != nil {
+			if err := c.gitAutoCommit(cmd, status); err != nil {
 				return err
 			}
 		}
@@ -1861,7 +1903,7 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	})
 
 	// Read the config file.
-	if annotations.hasTag(runsWithInvalidConfig) {
+	if annotations.hasTag(doesNotRequireValidConfig) {
 		if c.configFileAbsPathErr == nil {
 			_ = c.readConfig()
 		}
@@ -2730,8 +2772,7 @@ func newConfigFile(bds *xdg.BaseDirectorySpecification) ConfigFile {
 		},
 		Format: writeDataFormatJSON,
 		Git: gitCmdConfig{
-			Command:               "git",
-			CommitMessageTemplate: templates.CommitMessageTmpl,
+			Command: "git",
 		},
 		GitHub: gitHubConfig{
 			RefreshPeriod: 1 * time.Minute,
