@@ -155,8 +155,8 @@ func (c *Config) newDoctorCmd() *cobra.Command {
 		Long:    mustLongHelp("doctor"),
 		RunE:    c.runDoctorCmd,
 		Annotations: newAnnotations(
+			doesNotRequireValidConfig,
 			runsCommands,
-			runsWithInvalidConfig,
 		),
 	}
 
@@ -296,6 +296,7 @@ func (c *Config) runDoctorCmd(cmd *cobra.Command, args []string) error {
 			ifNotExist:  checkResultInfo,
 			versionArgs: []string{"--version"},
 			versionRx:   onepasswordVersionRx,
+			minVersion:  &onepasswordMinVersion,
 		},
 		&binaryCheck{
 			name:        "bitwarden-command",
